# Generated by Django 3.2 on 2023-02-02 08:18

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_orghierarchy", "0010_rename_user_editable"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datasource",
            name="user_editable_organizations",
            field=models.BooleanField(
                default=False, verbose_name="Organizations may be edited by users"
            ),
        ),
    ]
