# Generated by Django 2.2.8 on 2020-01-08 08:42

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("django_orghierarchy", "0006_add_user_editable_to_data_source"),
    ]

    operations = [
        migrations.AlterField(
            model_name="organization",
            name="level",
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name="organization",
            name="lft",
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name="organization",
            name="rght",
            field=models.PositiveIntegerField(editable=False),
        ),
    ]
