# Generated by Django 3.1.2 on 2021-03-17 15:38

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "django_orghierarchy",
            "0009_add_change_organization_regular_users_permission",
        ),
    ]

    operations = [
        migrations.RenameField(
            model_name="datasource",
            old_name="user_editable",
            new_name="user_editable_organizations",
        ),
    ]
